------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Martin Morlot
--                   Mail : martin.morlot@lpp.polytechnique.fr
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.numeric_std.ALL;
USE IEEE.std_logic_1164.ALL;

LIBRARY staging;
USE staging.general_purpose_LPP_JCP.ALL;

--! Une ALU : Arithmetic and logical unit, permettant de raliser une ou plusieurs opration

ENTITY ALU_LPP_JCP IS
  GENERIC(
    Arith_en   : INTEGER := 1;
    Logic_en   : INTEGER := 1;
    Input_SZ_1 : INTEGER := 16;
    Input_SZ_2 : INTEGER := 16;
    COMP_EN    : INTEGER := 0           -- 1 =>  No Comp
    );
  PORT(
    clk   : IN  STD_LOGIC;              --! Horloge du composant
    reset : IN  STD_LOGIC;              --! Reset general du composant
    ctrl  : IN  STD_LOGIC_VECTOR(2 DOWNTO 0);  --! Permet de slectionner la/les opration dsire
    comp  : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);  --! (set) Permet de complmenter les oprandes
    OP1   : IN  STD_LOGIC_VECTOR(Input_SZ_1-1 DOWNTO 0);  --! Premier Oprande
    OP2   : IN  STD_LOGIC_VECTOR(Input_SZ_2-1 DOWNTO 0);  --! Second Oprande
    RES   : OUT STD_LOGIC_VECTOR(Input_SZ_1+Input_SZ_2-1 DOWNTO 0)  --! Rsultat de l'opration
    );
END ALU_LPP_JCP;

--! @details Slection grace a l'entre "ctrl" :
--! Pause                    : IDLE     = 000
--! Multiplieur/Accumulateur : MAC      = 001
--! Multiplication           : MULT     = 010
--! Addition                 : ADD      = 011
--! Reset du MAC             : CLRMAC   = 100
ARCHITECTURE ar_ALU OF ALU_LPP_JCP IS

BEGIN

  arith : IF Arith_en = 1 GENERATE
    MACinst : MAC_LPP_JCP
      GENERIC MAP(Input_SZ_1, Input_SZ_2, COMP_EN)
      PORT MAP(clk, reset, ctrl(2), ctrl(1 DOWNTO 0), comp, OP1, OP2, RES);
  END GENERATE;

END ARCHITECTURE ar_ALU;
